from pygame import Rect
from resource import get_image
from utils import blit_tinted

class Animation(object):

	def __init__(self, image_name, num_strips, num_frames, border = 0, loop = False):
		self.num_strips = num_strips
		self.num_frames = num_frames
		self.images = get_image(image_name, border = border)
		#if bg:
		#	print "Animation:", image_name, "masks =", self.images.get_masks()
		#	self.images.set_colorkey(bg)
		w, h = self.images.get_size()
		self.frame_size = (w // num_frames, h // num_strips)
		self.loop = loop
		self.strip = 0
		self.frame = 0
		self.running = False
		self.direction = 1
	
	def draw_frame(self, surface, strip, frame, align, pos, tint = None):
		s = strip % self.num_strips
		f = frame % self.num_frames
		w, h = self.frame_size
		src = Rect(f * w, s * h, w, h)
		dst = Rect(src)
		setattr(dst, align, pos)
		if tint:
			blit_tinted(surface, self.images, dst.topleft, tint, src)
		else:
			surface.blit(self.images, dst, src)
	
	def draw(self, surface, align, pos):
		self.draw_frame(surface, self.strip, self.frame, align, pos)
	
	def begin_frame(self):
		if self.running:
			overflow, frame = divmod(self.frame + self.direction, self.num_frames)
			if overflow and not self.loop:
				self.running = False
				return
			self.frame = frame
	
	def at_first_frame(self):
		return self.frame == 0

	def at_last_frame(self):
		return self.frame == self.num_frames - 1

	def start_from_beginning(self):
		self.frame = 0
		self.resume_forward()
	
	def resume_forward(self):
		self.direction = 1
		self.running = True

	def start_from_end(self):
		self.frame = self.num_frames - 1
		self.resume_backward()
	
	def resume_backward(self):
		self.direction = -1
		self.running = True

	def go_to_beginning(self):
		self.frame = 0
		self.running = False
	
	def go_to_end(self):
		self.frame = self.num_frames - 1
		self.running = False
