from pygame.locals import *
from root import RootWidget
from palettes import make_back_palette, make_wall_palette, \
	make_floor_palette, make_gizmo_palette, make_object_palette
from level import Level, Back, Floor, Wall, Gizmo, Object
from editor_level_view import EditorLevelView
from palette_views import ToolsView, BackPaletteView, WallPaletteView, \
	FloorPaletteView, GizmoPaletteView, ObjectPaletteView
from controls import TextField
from dialogs import alert, input_text
from level_loading import load_level
from level_saving import save_level
from player import Player
from editor_constants import *
from edit_params import edit_item_params

default_level_size = (4, 13) # rows, cols

class Editor(RootWidget):

	def __init__(self, screen):
		RootWidget.__init__(self, screen)
		self.back_palette = make_back_palette()
		self.gizmo_palette = make_gizmo_palette()
		self.object_palette = make_object_palette()
		self.wall_palette = make_wall_palette()
		self.floor_palette = make_floor_palette()
		self.current_palette_item = None
		self.current_tool = TOOL_SELECT
		level_view = EditorLevelView(self, *default_level_size)
		tool_view = ToolsView(self)
		tool_view.rect.left = level_view.rect.right
		bp_view = BackPaletteView(self, 2, 6)
		bp_view.rect.topleft = tool_view.rect.bottomleft
		wp_view = WallPaletteView(self, 1, 10)
		wp_view.rect.topleft = bp_view.rect.bottomleft
		fp_view = FloorPaletteView(self, 2, 4)
		fp_view.rect.topleft = wp_view.rect.bottomleft
		gp_view = GizmoPaletteView(self, 2, 6)
		gp_view.rect.topleft = fp_view.rect.bottomleft
		op_view = ObjectPaletteView(self, 2, 4)
		op_view.rect.topleft = gp_view.rect.bottomleft
		self.add(level_view)
		self.add(tool_view)
		self.add(bp_view)
		self.add(wp_view)
		self.add(fp_view)
		self.add(gp_view)
		self.add(op_view)
		self.player = Player()
		self.set_level(Level(*default_level_size))
	
	def set_level(self, level):
		self.level = level
		level.set_player(self.player)

	def key_down(self, event):
		if event.cmd:
			key = event.key
			if key == K_l:
				self.load_level_cmd()
			elif key == K_s:
				self.save_level_cmd()
	
	def load_level_cmd(self):
		filename = input_text("Load Level:", 300)
		if filename:
			try:
				level = load_level(filename)
				level.filename = filename
				self.set_level(level)
			except EnvironmentError, e:
				alert("Unable to load level: %s" % e)
	
	def save_level_cmd(self):
		level = self.level
		filename = input_text("Save level as:", 300, level.filename)
		if filename:
			try:
				level.filename = filename
				save_level(level, filename)
			except EnvironmentError, e:
				alert("Unable to save level: %s" % e)

	def click_cell(self, zone, row, col):
		print "Editor.click_cell:", zone, row, col
		tool = self.current_tool
		level = self.level
		if tool == TOOL_SELECT:
			if zone == ZONE_BACK:
				self.edit_cell_params(row, col)
		elif tool == TOOL_ADD:
			item = self.current_palette_item.clone()
			if item:
				if zone == ZONE_BACK:
					if isinstance(item, Back):
						level.backs[row, col] = item
					elif isinstance(item, Gizmo):
						level.gizmos[row, col] = item
					elif isinstance(item, Object):
						level.objects[row, col] = item
				elif zone == ZONE_WALL and isinstance(item, Wall):
					level.walls[row, col] = item
				elif zone == ZONE_FLOOR and isinstance(item, Floor):
					level.floors[row, col] = item
		elif tool == TOOL_ERASE:
			if zone == ZONE_BACK:
				if level.objects[row, col]:
					level.objects[row, col] = None
				elif level.gizmos[row, col]:
					level.gizmos[row, col] = None
				elif level.backs[row, col]:
					level.backs[row, col] = None
			elif zone == ZONE_WALL:
				level.walls[row, col] = None
			elif zone == ZONE_FLOOR:
				level.floors[row, col] = None

	def edit_cell_params(self, row, col):
		items = [self.level.gizmo_at(row, col), self.level.object_at(row, col)]
		edit_item_params(items)

		