from resource import resource_path

registered_classes = {}

class Params(object):

	def __init__(self, **kwds):
		self.__dict__.update(kwds)

	def __str__(self):
		items = ["%s=%r" % item for item in self.__dict__.items()]
		return ",".join(items)


class Saveable(object):

	is_static = True

	def __init__(self, **kwds):
		self.set_params(Params(**kwds))
	
	def get_params(self):
		return Params()
	
	def set_params(self, params):
		pass
	
	def clone(self):
		if self.is_static:
			return self
		else:
			return instantiate_class(self.__class__, self.get_params())


def level_path(filename):
	if not filename.endswith(".lvl"):
		filename = filename + ".lvl"
	return resource_path("levels", filename)

def register_class(cls):
	registered_classes[cls.__name__] = cls

def instantiate_class(cls, params):
	#print "Instantiating", cls, "with", params
	x = cls.__new__(cls)
	x.set_params(params)
	return x

def instantiate(class_name, params):
	cls = registered_classes.get(class_name)
	if cls:
		return instantiate_class(cls, params)
